// $Id: NumericAxis.java 10992 2009-10-05 11:30:01Z mike $

package org.faceless.graph2;

import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.util.Locale;

/**
 * The most common type of Axis, a NumericAxis is used to display numbers, either
 * as integers, real numbers, percentages, currency amounts or any type of format
 * that can be created with a {@link NumberFormat}.
 */
public class NumericAxis extends Axis
{
    private NumberFormat f;
    private final int density;
    private boolean stretch, integer;

    static NumberFormat getIntegerInstance() {  // Def'd here because not in 1.3
        NumberFormat format = NumberFormat.getInstance();
        format.setMaximumFractionDigits(0);
        if (format instanceof DecimalFormat) {
            ((DecimalFormat)format).setDecimalSeparatorAlwaysShown(false);
        }
        format.setParseIntegerOnly(true);
        return format;
    }

    /**
     * Return a NumericAxis that formats it's values as integers
     */
    public static final NumericAxis getIntegerAxis() {
        return new NumericAxis(getIntegerInstance());
    }

    /**
     * Return a NumericAxis that formats it's values as real numbers
     */
    public static final NumericAxis getFloatingPointAxis() {
        return new NumericAxis(NumberFormat.getNumberInstance());
    }

    /**
     * Return a NumericAxis that formats it's values as currencies in
     * the specified locale.
     * @param locale which Locale to use for currencies, or <code>null</code> to use the default locale
     */
    public static final NumericAxis getCurrencyAxis(Locale locale) {
        return new NumericAxis(NumberFormat.getCurrencyInstance(locale==null ? Locale.getDefault() : locale));
    }

    /**
     * Create a new NumericAxis with the specified format and the default density.
     * @param format the format to use to display the values
     */
    public NumericAxis(NumberFormat format) {
        this(format, DENSITY_NORMAL);
    }

    /**
     * Create a new NumericAxis with the specified format and densty
     * @param format the format to use to display the values
     * @param density the density to use - usually one of {@link Axis#DENSITY_NORMAL}, {@link Axis#DENSITY_SPARSE} or {@link Axis#DENSITY_MINIMAL}, but may be any integer which is roughly the number of intended teeth on the axis.
     */
    public NumericAxis(NumberFormat format, int density) {
        setFormat(format);
        this.density = density;
        stretch = true;
    }

    void setFormat(NumberFormat format) { f=format;
        this.integer = format==null ? false : format(1).equals(format(1.4));
    }

    boolean isInteger() {
        return integer;
    }

    public String format(double in) {
        if (in==-0.0) in=0;
        synchronized(f) {
            return f.format(in);
        }
    }

    /**
     * Whether to "stretch" the ends of the graph to a "useful" value or
     * not. For instance, when this setting is on, plotting values from
     * 0 to 98 will result in an axis that runs from 0 to 100. Turning
     * it off means the top of the graph stops at exactly 98. This
     * setting is on by default
     * @param stretch whether to stretch the ends of the graph
     */
    public void setStretchEnds(boolean stretch) {
        this.stretch = stretch;
    }

    /**
     * <p>
     * Which steps between min and max should be marked with a tooth.
     * This method returns the default settings, which is to calculate
     * a number of "useful" values between min and max, possibly rounding
     * those two values up or down to fit the scale. The number of values
     * returned depends on the density setting.
     * </p>
     */
    public double[] steps(double min, double max) {
        double[] out;
        double ss = stepSize(min, max);
        if (integer) ss = Math.ceil(ss);

        if (ss==0) {
            out = new double[1];
            out[0] = min;
        } else {
            double newmin = Math.floor(min/ss)*ss;
            double newmax = Math.ceil(max/ss)*ss;
            if (!stretch) {
                if (newmin+ss-min < min-newmin) newmin+=ss;
                if (max-newmax+ss < newmax-max) newmax-=ss;
            }
            int num = (int)Math.floor((newmax+(ss/5)-newmin)/ss);
            out = new double[num+1];
            for (int i=0;i<out.length;i++) {
                out[i] = newmin+(ss*i);
                if (integer) out[i] = Math.floor(out[i]);
            }
            if (!stretch) {
                out[0] = min;
                out[out.length-1] = max;
            }
        }
        return out;
    }

    private static final double diffs[] = { 0, 1, 2, 5, 5, 5, 5, 5, 10, 10, 10 };

    private double stepSize(double min, double max) {
        double diff = Math.abs(max-min);
        if (diff==0) return 0;
        diff/=density;

        double mul=1;
        while (diff<1) {
            mul *= 10;
            diff *= 10;
        }

        while (diff>10) {
            mul /= 10;
            diff /= 10;
        }

        diff = Math.round(diff);
        diff = diffs[(int)diff]/mul;

        return diff;
    }
}
